#include                <string.h>
#include                <tap.h>
#include                "libFireBird.h"
#include                "RemoteSwitch.h"

#include                "graphic/key_progdown.gd"
#include                "graphic/key_progup.gd"
#include                "graphic/key_ok.gd"
#include                "graphic/key_exit.gd"
#include                "graphic/icon_red.gd"

#define PROGRAM_NAME    "RemoteSwitchGUI"

TAP_ID                  (0x8E0A423D);
TAP_PROGRAM_NAME        (PROGRAM_NAME " " VERSION);
TAP_AUTHOR_NAME         ("FireBird");
TAP_DESCRIPTION         ("Modifies the IR settings of the front panel");
TAP_ETCINFO             (__DATE__);

/*****************************************************************************************************************************/
/* Prototypes                                                                                                                */
/*****************************************************************************************************************************/
void LoadINI(void);
void SaveINI(void);
void UpdateIrDevice(void);
void DrawMainWindow(void);
void ToggleMode(dword SelMode);

bool                    Mode[NRRCUMODES + 1];  //Mode[0] is not used
bool                    MainWindow = FALSE;
REMOTE_TYPE             SelectedRemoteType = RT_2100;
bool                    DoNotReenter = FALSE;

typedef enum
{
  DIALOGITEM_Mode1,
  DIALOGITEM_Mode2,
  DIALOGITEM_Mode3,
  DIALOGITEM_Mode4,
  DIALOGITEM_Mode5,
  DIALOGITEM_Mode6,
  DIALOGITEM_UsedRemote,
  DIALOGITEM_Separator,
  DIALOGITEM_SaveAndExit,
  DIALOGITEM_Exit,
  DIALOGITEM_NRITEMS
}tDialogItems;

typedef enum
{
  LS_Cancel,
  LS_Save,
  LS_RCU,
  LS_Max4Warning,
  LS_Modify,
  LS_NrStrings
} tLanguageStrings;

void CreateRootDir(void)
{
  //Check & Create Folders
  HDD_TAP_PushDir();
  HDD_ChangeDir("/ProgramFiles");
  if(!TAP_Hdd_Exist("Settings")) TAP_Hdd_Create("Settings", ATTR_FOLDER);
  HDD_ChangeDir("Settings");
  if(!TAP_Hdd_Exist("RemoteSwitch")) TAP_Hdd_Create("RemoteSwitch", ATTR_FOLDER);
  HDD_TAP_PopDir();
}

void LoadINI(void)
{
  dword                 i;
  char                  Key[8];
  INILOCATION           INILocation;

  TAP_Hdd_ChangeDir(ROOTDIR);
  for(i = 1; i <= NRRCUMODES; i++)
    Mode[i] = TRUE;

  INILocation = INIOpenFile(INIFILENAME, NULL);
  if (INILocation == INILOCATION_NewFile)
  {
    INICloseFile();
  }
  else
  {
    for(i = 1; i <= NRRCUMODES; i++)
    {
      TAP_SPrint(Key, "Mode%d", i);
      Mode[i] = INIGetInt(Key, 1, 0, 1) != 0;
    }
    SelectedRemoteType = INIGetInt("SelectedRC", 0, 0, 3);

    INICloseFile();
  }
  SaveINI();
}

void SaveINI(void)
{
  dword                 i;
  char                  Key[8];
  TAPCOM_Channel        Channel;

  TAP_Hdd_ChangeDir(ROOTDIR);
  INIOpenFile(INIFILENAME, PROGRAM_NAME);
  for(i = 1; i <= NRRCUMODES; i++)
  {
    TAP_SPrint(Key, "Mode%d", i);
    INISetInt(Key, Mode[i] ? 1 : 0);
  }
  INISetInt("SelectedRC", SelectedRemoteType);
  INISaveFile(INIFILENAME, INILOCATION_AtCurrentDir, NULL);
  INICloseFile();

  Channel = TAPCOM_OpenChannel(0x8E0A4215, TAPCOM_ReloadINI, 0, NULL);
  TAPCOM_CloseChannel(Channel);
}

void UpdateIrDevice(void)
{
  int i, Index;

  //Deactivate all filters
  for(Index = 0; Index < 4; Index++)
  {
    SetRemoteMode(1, Index, FALSE);
  }

  //Now program every active mode
  Index = 0;
  for(i = 1; i <= NRRCUMODES; i++)
  {
    if(Mode[i])
    {
      SetRemoteMode(i, Index, TRUE);
      Index++;
    }
  }
}

char *RemoteToString(REMOTE_TYPE RemoteType)
{
  switch(RemoteType)
  {
    case RT_5000:     return "5000, 5000MP";
    case RT_2100:     return "2100, 2400";
    case RT_7100PLUS: return "7100+, 2401";
    case RT_7260PLUS: return "7260+";
    default: return "???";
  }
}

void DrawMainWindow (void)
{
  OSDMenuInitialize(FALSE, TRUE, FALSE, TRUE, PROGRAM_NAME, VERSION);

  OSDMenuItemAdd("Mode 1 (Masterpiece, 2401)", Mode[1] ? "active" : "inactive", NULL, NULL, TRUE, TRUE, 0);
  OSDMenuItemAdd("Mode 2 (\?\?\?)",            Mode[2] ? "active" : "inactive", NULL, NULL, TRUE, TRUE, 0);
  OSDMenuItemAdd("Mode 3 (\?\?\?)",            Mode[3] ? "active" : "inactive", NULL, NULL, TRUE, TRUE, 0);
  OSDMenuItemAdd("Mode 4 (TF5000, TF7700)",    Mode[4] ? "active" : "inactive", NULL, NULL, TRUE, TRUE, 0);
  OSDMenuItemAdd("Mode 5 (2100, 2410)",        Mode[5] ? "active" : "inactive", NULL, NULL, TRUE, TRUE, 0);
  OSDMenuItemAdd("Mode 6 (TRF-7260)",          Mode[6] ? "active" : "inactive", NULL, NULL, TRUE, TRUE, 0);

  OSDMenuItemAdd(LangGetString(LS_RCU), RemoteToString(SelectedRemoteType), NULL, NULL, TRUE, TRUE, 0);

  OSDMenuButtonAdd(1, BI_Red, NULL, LangGetString(LS_Cancel));
  OSDMenuButtonAdd(1, BI_Green, NULL, LangGetString(LS_Save));
  OSDMenuButtonAdd(2, BI_Exit, NULL, LangGetString(LS_Cancel));

  MainWindow = TRUE;
}

void ToggleMode(dword SelMode)
{
  if(SelMode <= NRRCUMODES)
  {
    Mode[SelMode] = !Mode[SelMode];
    OSDMenuItemModifyValue(OSDMenuGetCurrentItem(), Mode[SelMode] ? "active" : "inactive");
    OSDMenuUpdate(FALSE);
  }
}

void ToggleRemoteType(bool Up)
{
  if(Up)
  {
    SelectedRemoteType += 1;
    if(SelectedRemoteType > RT_7260PLUS) SelectedRemoteType = RT_5000;
  }
  else
  {
    SelectedRemoteType -= 1;
    if(SelectedRemoteType > RT_7260PLUS) SelectedRemoteType = RT_7260PLUS;
  }

  OSDMenuItemModifyValue(DIALOGITEM_UsedRemote, RemoteToString(SelectedRemoteType));

  OSDMenuUpdate(FALSE);
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
  if(OSDMenuMessageBoxIsVisible() || OSDMenuInfoBoxIsVisible()) OSDMenuEvent(&event, &param1, &param2);

  if(DoNotReenter) return param1;
  DoNotReenter = TRUE;

  if(!MainWindow)
  {
    DrawMainWindow();
    OSDMenuUpdate(FALSE);
  }

  if((event != EVT_KEY) && (event != EVT_IDLE))
  {
    DoNotReenter = FALSE;
    return param1;
  }

  if(event == EVT_KEY)
  {
    OSDMenuEvent(&event, &param1, &param2);
    switch (param1)
    {
      case RKEY_Right:
      {
        if(OSDMenuGetCurrentItem() < DIALOGITEM_UsedRemote)
          ToggleMode(OSDMenuGetCurrentItem() + 1);
        else if(OSDMenuGetCurrentItem() == DIALOGITEM_UsedRemote)
          ToggleRemoteType(TRUE);
        break;
      }

      case RKEY_Left:
      {
        if(OSDMenuGetCurrentItem() < DIALOGITEM_UsedRemote)
          ToggleMode(OSDMenuGetCurrentItem() + 1);
        else if(OSDMenuGetCurrentItem() == DIALOGITEM_UsedRemote)
          ToggleRemoteType(FALSE);
        break;
      }

      case RKEY_Red:
      {
        LoadINI();
        UpdateIrDevice();
        OSDMenuDestroy();
        TAP_Exit();
        break;
      }

      case RKEY_Green:
      case RKEY_Ok:
      {
        int i, j;

        j = 0;
        for(i = 1; i <= NRRCUMODES; i++)
          if(Mode[i]) j++;

        if(j > 4)
        {
          OSDMenuMessageBoxInitialize(PROGRAM_NAME, LangGetString(LS_Max4Warning));
          OSDMenuMessageBoxButtonAdd(LangGetString(LS_Modify));
          OSDMenuMessageBoxShow();
          while(OSDMenuMessageBoxIsVisible())
          {
            TAP_SystemProc();
          }
          break;
        }
        SaveINI();
        UpdateIrDevice();
        //don't break
      }

      case RKEY_Exit:
      case RKEY_Menu:
      {
        OSDMenuDestroy();
        LangUnloadStrings();
        TAP_Exit();
        break;
      }

      default:
        ;
    }
  }

  DoNotReenter = FALSE;
  return 0;
}

int TAP_Main(void)
{
  char s[128];

  CreateRootDir();
  LoadINI();
  UpdateIrDevice();
  KeyTranslate(TRUE, &TAP_EventHandler);

  if(!LangLoadStrings(LNGFILENAME, LS_NrStrings, LAN_English, "RemoteSwitch"))
  {
    DoNotReenter = TRUE;
    TAP_SPrint(s, "Language file '%s' not found!", LNGFILENAME);
    OSDMenuInfoBoxShow(PROGRAM_NAME " " VERSION, s, 500);
    do
    {
      TAP_SystemProc();
    }while(OSDMenuInfoBoxIsVisible());
    OSDMenuInfoBoxDestroy();

    return 0;
  }

  return 1;
}
