#include                "tap.h"
#include                "LibFireBird.h"
#include                "RemoteSwitch.h"

#define PROGRAM_NAME    "VolKeyPatch"
#define LOGFILENAME     "VolKeyPatch.log"

TAP_ID                  (0x8E0A4215);
TAP_PROGRAM_NAME        (PROGRAM_NAME" "VERSION);
TAP_AUTHOR_NAME         ("FireBird");
TAP_DESCRIPTION         ("Maps the left/right keys of the old remote to vol up/down");
TAP_ETCINFO             (__DATE__);

REMOTE_TYPE             SystemRemoteType = RT_2100;
REMOTE_TYPE             SelectedRemoteType = RT_5000;
dword                   TranslationType = 0;
char                    Log[512];

void CreateRootDir(void)
{
  //Check & Create Folders
  HDD_TAP_PushDir();
  HDD_ChangeDir("/ProgramFiles");
  if(!TAP_Hdd_Exist("Settings")) TAP_Hdd_Create("Settings", ATTR_FOLDER);
  HDD_ChangeDir("Settings");
  if(!TAP_Hdd_Exist("RemoteSwitch")) TAP_Hdd_Create("RemoteSwitch", ATTR_FOLDER);
  HDD_TAP_PopDir();
}

void WriteLog(char *s)
{
  HDD_TAP_PushDir();
  TAP_Hdd_ChangeDir(ROOTDIR);
  LogEntry(LOGFILENAME, PROGRAM_NAME, TRUE, TIMESTAMP_YMDHMS, s);
  HDD_TAP_PopDir();
}

char *RemoteToString(REMOTE_TYPE RemoteType)
{
  switch(RemoteType)
  {
    case RT_5000:     return "5000, 5000MP";
    case RT_2100:     return "2100, 2400";
    case RT_7100PLUS: return "7100+, 2401";
    case RT_7260PLUS: return "7260+";
    default: return "???";
  }
}

void LoadINI(void)
{
  INILOCATION           INILocation;

  TAP_Hdd_ChangeDir(ROOTDIR);

  INILocation = INIOpenFile(INIFILENAME, "RemoteSwitch");
  if (INILocation == INILOCATION_NewFile)
  {
    INICloseFile();
  }
  else
  {
    SelectedRemoteType = INIGetInt("SelectedRC", 0, 0, 3);
    INICloseFile();
  }

  //Supported translation types:
  //0x0001: TF5000 RC -> SRP2100
  //0x0002: TF5000 RC -> TF7100+
  //0x0102: SRP2100 RC -> TF7100+
  TranslationType = (SelectedRemoteType << 8) | SystemRemoteType;

  //Catch all unsupported combinations
  switch(TranslationType)
  {
    case 0x0100:  //SRP2100 RC -> TF5000
    case 0x0200:  //TF7100+ RC -> TF5000
    case 0x0201:  //TF7100+ RC -> SRP2100
    {
      TAP_SPrint(Log, "Unsupported combination 0x%4.4x.", TranslationType);
      WriteLog(Log);
    }

    case 0x0000:
    case 0x0101:
    case 0x0202:
    {
      WriteLog("Selected remote type and system remote type are identical. Nothing to do.");
    }

    default:
    {
      TAP_SPrint(Log, "Remote = '%s', target = '%s'", RemoteToString(TranslationType >> 8), RemoteToString(TranslationType & 0xff));
      WriteLog(Log);
    }
  }
}

bool (*Orig_PutKeyEvt)(dword a1);

bool Hooked_PutKeyEvt(dword KeyParam)
{
  dword                 KeyFlags;
  dword                 State, SubState;

  KeyFlags = KeyParam & 0xfffe0000;
  KeyParam = KeyParam & 0x0001ffff;

  TAP_GetState(&State, &SubState);

  switch(TranslationType)
  {
    case 0x0001:  //TF5000 RC -> SRP2100
    {
      if((State == STATE_Normal) && (SubState == SUBSTATE_Normal))
      {
        if(KeyParam == RKEY_Right)        KeyParam = RKEY_VolUp;    //Vol+
        else if(KeyParam == RKEY_Left)    KeyParam = RKEY_VolDown;  //Vol-
        else if(KeyParam == RKEY_Up)      KeyParam = RKEY_ChUp;     //Ch+
        else if(KeyParam == RKEY_Down)    KeyParam = RKEY_ChDown;   //Ch-
      }
      else
      {
        if(KeyParam == RKEY_VolUp)        KeyParam = RKEY_Right;    //Vol+
        else if(KeyParam == RKEY_VolDown) KeyParam = RKEY_Left;     //Vol-
      }

      if((KeyParam == RKEY_Ab) && (SubState != SUBSTATE_PvrPlayingSearch)) KeyParam = 0x10049;  //White (Option)

      break;
    }

    case 0x0002:  //TF5000 RC -> TF7100+
    {
      if(KeyParam == 0x1000d)             KeyParam = RKEY_Fav;        //Fav
      else if(KeyParam == RKEY_Bookmark)  KeyParam = RKEY_VolDown;    //Green
      else if(KeyParam == RKEY_Goto)      KeyParam = 0x10049;         //Yellow
      else if(KeyParam == RKEY_Check)     KeyParam = 0x1004a;         //Blue
      else if(KeyParam == RKEY_NewF1)     KeyParam = RKEY_VolUp;      //Red
      else if(KeyParam == 0x10049)        KeyParam = RKEY_Ab;         //AB-Cut, White
      else
      {
        if((State == STATE_Normal) && (SubState == SUBSTATE_Normal))
        {
          if(KeyParam == RKEY_Left)       KeyParam = RKEY_Bookmark;   //Vol-
          else if(KeyParam == RKEY_Up)    KeyParam = RKEY_Goto;       //Ch+
          else if(KeyParam == RKEY_Right) KeyParam = RKEY_NewF1;      //Vol+
          else if(KeyParam == RKEY_Down)  KeyParam = RKEY_Check;      //Ch-
        }
      }
      break;
    }

    case 0x0102:  //SRP2100 RC -> TF7100+
    {
      if(KeyParam == RKEY_ChUp)           KeyParam = 0x10049;         //Yellow
      else if(KeyParam == RKEY_ChDown)    KeyParam = 0x1004a;         //Blue
      else if(KeyParam == 0x10049)        KeyParam = 0x1003e;         //AB-Cut
      break;
    }
  }

  return Orig_PutKeyEvt(KeyParam | KeyFlags);
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
  (void) param2;

  if(event == EVT_STOP && param1 < 2)
  {
    UnhookFirmware("_Z9PutKeyEvtj", Hooked_PutKeyEvt, (void*)&Orig_PutKeyEvt);
    TAP_Exit();
  }

  if(event == EVT_TAPCOM)
  {
    TAPCOM_Channel  Channel;
    dword           ServiceID;

    Channel = TAPCOM_GetChannel(param1, NULL, &ServiceID, NULL, NULL);
    if(ServiceID == TAPCOM_ReloadINI) LoadINI();
    TAPCOM_Finish(Channel, 0);
  }

  return param1;
}

int TAP_Main(void)
{
  dword                 i;
  tToppyInfo           *ToppyInfo;
  tFWDATHeader         *FWDatHeader;

  CreateRootDir();

  //Get toppy information
  if(LoadFirmwareDat(&FWDatHeader, &ToppyInfo, NULL))
  {
    for (i = 0; i < FWDatHeader->NrOfToppyInfoEntries; i++, ToppyInfo++)
    {
      if(ToppyInfo->SysID == TAP_GetSystemId())
      {
        SystemRemoteType = ToppyInfo->RemoteType;
        break;
      }
    }
  }
  else
  {
    WriteLog("Failed to load FirmwareTMS.dat");
  }

  //For debugging only
  dword                *got;
  dword                 FirmwareFunction;

  FirmwareFunction = TryResolve("_Z9PutKeyEvtj");
  if(FirmwareFunction)
  {
    TAP_SPrint(Log, "Located _Z9PutKeyEvtj @ 0x%8.8x", FirmwareFunction);
    WriteLog(Log);
    got = FindGotPointer(FirmwareFunction);
    if(got)
    {
      TAP_SPrint(Log, "Located the .got entry @ %8.8p", got);
      WriteLog(Log);
    }
    else
    {
      WriteLog("Failed to locate the .got entry of _Z9PutKeyEvtj");
    }
  }
  else
  {
    WriteLog("Failed to resolve _Z9PutKeyEvtj");
  }


  //Hook the key event sink
  if(!HookFirmware("_Z9PutKeyEvtj", Hooked_PutKeyEvt, (void*)&Orig_PutKeyEvt))
  {
    WriteLog("Failed to hook PutKeyEvt()");
    return 0;
  }

  LoadINI();

  return 1;
}
